@@ -28,8 +28,6 @@
 
 #include "hb-private.hh"
 
-#include "hb-version.h"
-
 #include "hb-mutex-private.hh"
 #include "hb-object-private.hh"
 
@@ -236,6 +234,7 @@ struct hb_language_item_t {
 
 static hb_language_item_t *langs;
 
+#ifdef HAVE_ATEXIT
 static inline
 void free_langs (void)
 {
@@ -246,6 +245,7 @@ void free_langs (void)
     langs = next;
   }
 }
+#endif
 
 static hb_language_item_t *
 lang_find_or_insert (const char *key)
@@ -292,11 +292,13 @@ retry:
 hb_language_t
 hb_language_from_string (const char *str, int len)
 {
+  char strbuf[64];
+
   if (!str || !len || !*str)
     return HB_LANGUAGE_INVALID;
 
-  if (len >= 0) {
-    char strbuf[64];
+  if (len >= 0)
+  {
     len = MIN (len, (int) sizeof (strbuf) - 1);
     str = (char *) memcpy (strbuf, str, len);
     strbuf[len] = '\0';
